#include <iostream>
#include <algorithm>
#include <vector>
#include <map>
#include <set>
#include <cassert>
#include <cmath>
#include <numeric>

using namespace std;

typedef long long ll;

constexpr int N = 200000;
ll x[N], y[N], r[N];

bool a_in_b(int a, int b) {
    return r[a] < r[b] &&
        (x[a] - x[b]) * (x[a] - x[b]) + (y[a] - y[b]) * (y[a] - y[b]) <= (r[b] - r[a]) * (r[b] - r[a]);
}

struct ArcComp {
    bool operator()(const pair<int, bool>& left, const pair<int, bool>& right) const {
        const auto& [i1, d1] = left;
        const auto& [i2, d2] = right;

        if (i1 == i2)
            return d1 < d2;

        if (a_in_b(i1, i2)) {
            return d2;
        }
        if (a_in_b(i2, i1)) {
            return d1 == 0;
        }

        return (y[i1] < y[i2]);
    }
};

set<pair<int, bool>, ArcComp> arcs;

map<pair<ll, ll>, int> inner_touch_vecs[N];

set<pair<int, int>> outer_touch_pairs;

bool outer_touch(int a, int b) {
    return (x[a] - x[b]) * (x[a] - x[b]) + (y[a] - y[b]) * (y[a] - y[b]) == (r[b] + r[a]) * (r[b] + r[a]);
}

bool inner_touch_a_in_b(int a, int b) {
    return (r[a] < r[b]) && (x[a] - x[b]) * (x[a] - x[b]) + (y[a] - y[b]) * (y[a] - y[b]) == (r[b] - r[a]) * (r[b] - r[a]);
}

pair<ll, ll> dir(int from, int to) {
    assert(from != to);
    ll g = abs(gcd(x[to] - x[from], y[to] - y[from]));
    //cout << from << ' ' << to << ' ' << x[from] << ' ' << x[to] << ' ' << y[from] << ' ' << y[to] << endl;
    assert(g != 0);
    return {(x[to] - x[from]) / g, (y[to] - y[from]) / g};
}

void check_touch(int a, int b) {
    //cout << "check touch start" << endl;
    if (a == b) return;
    if (y[a] == y[b]) return; // hotizontal touches separate
    //cout << "check touch " << a << ' ' << b << endl;
    if(outer_touch(a, b)) {
        outer_touch_pairs.emplace(a, b);
    }
    if(inner_touch_a_in_b(a, b)) {
        inner_touch_vecs[b][dir(b, a)] = a;
    }
    if(inner_touch_a_in_b(b, a)) {
        inner_touch_vecs[a][dir(a, b)] = b;
    }
    //cout << "check touch done" << endl;
}

void add_arc(pair<int, bool> arc) {
    //cout << "add start " << arc.first << "." << arc.second << endl;
    auto next = arcs.upper_bound(arc);
    if (next != arcs.end()) {
        check_touch(arc.first, next->first);
    }
    if (next != arcs.begin()) {
        next--;
        if (next != arcs.end())
            check_touch(arc.first, next->first);
    }
    arcs.insert(arc);
    //cout << "add done" << endl;
}

void del_arc(pair<int, bool> arc) {
    //cout << "del start " << arc.first << "." << arc.second << endl;
    arcs.erase(arc);
    auto next = arcs.upper_bound(arc);
    if (next != arcs.end() && next != arcs.begin()) {
        auto prev = next;
        --prev;
        check_touch(prev->first, next->first);
    }
    //cout << "del done" << endl;
}

ll count_inner_touches(int a, pair<ll, ll> dir) {
    int ans = 1;
    while (inner_touch_vecs[a].find(dir) != inner_touch_vecs[a].end())
        ans++, a = inner_touch_vecs[a][dir];
    return ans;
}

void solve() {
    int n; cin >> n;
    for (int i = 0; i < n; i++)
        cin >> x[i] >> y[i] >> r[i];

    {
        for (int i = 0; i < n; i++) {
            inner_touch_vecs[i].clear();
        }
        arcs.clear();
        outer_touch_pairs.clear();
    }

    map<pair<int, int>, ll> lpoints, rpoints;
    map<ll, vector<int>> add, del;
    for (int i = 0; i < n; i++) {
        add[x[i] - r[i]].push_back(i);
        lpoints[{x[i] - r[i], y[i]}]++;
        rpoints[{x[i] + r[i], y[i]}]++;
        del[x[i] - r[i]].push_back(i);
    }

    ll hor_ans = 0;
    for (auto& [p, k] : lpoints) {
        hor_ans += lpoints[p] * rpoints[p];
    }

    //cout << "hor = " << hor_ans << endl;

    //cout << "HERE" << endl;

    map<ll, vector<int>> add_arcs;
    map<ll, vector<int>> del_arcs;
    set<ll> pts;
    for (int i = 0; i < n; i++) {
        pts.insert(x[i] - r[i]);
        add_arcs[x[i] - r[i]].push_back(i);
        pts.insert(x[i] + r[i]);
        del_arcs[x[i] + r[i]].push_back(i);
    }
    //cout << "HERE" << endl;

    

    for (int x : pts) {
        for (int i : del_arcs[x]) {
            del_arc({i, false});
            del_arc({i, true});
        }
        for (int i : add_arcs[x]) {
            add_arc({i, false});
            add_arc({i, true});
        }
    }

    //cout << "HERE" << endl;

    ll ans = hor_ans;
    for (auto [a, b] : outer_touch_pairs) {
        //cout << "for " << a << " to (" << dir(b, a).first << " " << dir(a, b).second << ")" << endl;
        //cout << "    " << count_inner_touches(a, dir(a, b)) << endl;
        ans += count_inner_touches(a, dir(a, b)) * count_inner_touches(b, dir(b, a));
    }
    //cout << "ans = ";
    cout << ans << '\n';
}

int main() {

    ios_base::sync_with_stdio(0);
    cin.tie(0);
    int t; cin >> t;
    while (t--) {
        solve();
    }
}
